/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/newh/tadt/memory.h
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#ifndef tadt_memory
#define tadt_memory

/*  
*  Routine: tadt_init
*
*  Description: Allocate globals shared in the tadt system.
*               These globals are shared variables such as seconds structures.
*
*  Arguments:   None
*
*  Return Value: None
*
*  Errors: None
*  
*  Side Effects: overwrites the tadt global pointers
*/

void tadt_init();

/*  
*  Routine: tadt_malloc
*
*  Description: Allocate storage from the TADT memory heap.  Marks a region
*               of mem_size bytes which is now in use.  If no more memory
*               can be allocated, the behaviour of this function is undefined.
*
*  Arguments: size -- (IN): the number of bytes to allocate
*
*  Return Value: The address of allocated memory
*
*  Errors: None
*  
*  Side Effects: Some of the TADT heap is marked as in use. This may lead
*                to memory leaks later on.
*/

void *tadt_malloc(int size);

/*  
*  Routine:  tadt_free
*
*  Description: Frees previously allocated storage
*
*  Arguments: address -- (IN):  address of previously allocated storage
*
*  Return Value: TRUE if the address was de-allocated, FALSE otherwise
*
*  Errors: None
*
*  Side Effects: If the address has not been allocated previously, then
*                this will could lead to internal errors.
*/

bool tadt_free(void *address);

/* 
*  Routine: tadt_allocate_e
*
*  Description: Memory allocation for an event
*
*  Arguments: resolution -- (IN): which resolution to allocate
*
*  Return Value: Address of allocated event
* 
*  Errors: None
*
*  Side Effects: Mallocs space
*/

event_type tadt_allocate_e(resolution_type resolution);

/* 
*  Routine: tadt_allocate_s
*  
*  Description: Memory allocation for an span
*
*  Arguments: resolution -- (IN): which resolution to allocate
*
*  Return Value: Address of allocated span
* 
*  Errors: None
*
*  Side Effects: Mallocs space
*/

span_type tadt_allocate_s(resolution_type resolution);

/* 
*  Routine: tadt_allocate_i
*  
*  Description: Memory allocation for an interval
*
*  Arguments: resolution1 -- (IN): resolution of beginning event
*                                  resolution of the ending event
*
*  Return Value: Address of allocated interval
* 
*  Errors: None
*
*  Side Effects: Mallocs space
*/

interval_type tadt_allocate_i(resolution_type resolution1, resolution_type resolution2);

/* 
*  Routine: tadt_allocate_ts   
*  
*  Description: Memory allocation for an time-stamp.  Allocates the max space.
*
*  Arguments: None
*
*  Return Value: Address of allocated time-stamp
* 
*  Errors: None
*
*  Side Effects: Mallocs space
*/

timestamp_type tadt_allocate_ts();

/* 
*  Routine: tadt_allocate_vs_struct
*  
*  Description: Memory allocation for an variable span structure.
*
*  Arguments: None
*
*  Return Value: Address of allocated vs_struct
* 
*  Errors: None
*
*  Side Effects: Mallocs space
*/

vs_type tadt_allocate_vs_struct();

/* 
*  Routine: tadt_allocate_avg_state_e
*  
*  Description: Memory allocation for an average event state structure.
*
*  Arguments: None
*
*  Return Value: Address of allocated average event state structure
* 
*  Errors: None
*
*  Side Effects: Mallocs space
*/

avg_state_e tadt_allocate_avg_state_e();

/* 
*  Routine: tadt_allocate_avg_state_fs
*  
*  Description: Memory allocation for an average fixed span state structure.
*
*  Arguments: None
*
*  Return Value: Address of allocated average fixed span state structure
* 
*  Errors: None
*
*  Side Effects: Mallocs space
*/

avg_state_fs tadt_allocate_avg_state_fs();

/* 
*  Routine: tadt_allocate_string_struct
*  
*  Description: Memory allocation for string structure
*
*  Arguments: None
*
*  Return Value: Address of allocated string structure
* 
*  Errors: None
*
*  Side Effects: Mallocs space
*/

string_struct tadt_allocate_string_struct(int capacity);

/* 
*  Routine: tadt_allocate_seconds
*  
*  Description: Memory allocation for seconds structure
*
*  Arguments: None
*
*  Return Value: Address of allocated seconds structure
* 
*  Errors: None
*
*  Side Effects: Mallocs space
*/

seconds_type tadt_allocate_seconds();

/* tadt internal helper routines to quickly allocate events */
void reset_quick_space_allocation();
event_type quickly_allocate_event();

#endif
