/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/newh/tadt/timestamp.h
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#ifndef tadt_timestamp
#define tadt_timestamp

/*
*  Routine:  tadt_repack_e
*
*  Description:  Repacks an event timestamp from the unpacked format.
*                We assume enough space has been allocated.  Always
*                repacks to extended resolution.  Use coercing to 
*                fit into a smaller space.
*
*  Arguments: event -- (IN): event structure
*             result -- (OUT): result event
*
*  Return Value: Error code
*
*  Errors: none
*
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_repack_e(event_type event, event_type result);

/*
*  Routine:  tadt_repack_fs
*
*  Description:  Repacks a span timestamp from the unpacked format.
*                We assume enough space has been allocated.  Always
*                repacks to extended resolution.  Use coercing to 
*                fit into a smaller space.
*
*  Arguments: span -- (IN): span structure
*             result -- (OUT): result span
*
*  Return Value: Error code
*
*  Errors: none
*
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_repack_fs(span_type span, span_type result);

/*
*  Routine:  tadt_repack_i
*
*  Description:  Repacks an interval timestamp from the unpacked format.
*                We assume enough space has been allocated.  Always
*                repacks to extended resolution.  Use coercing to 
*                fit into a smaller space.
*
*  Arguments: interval -- (IN): interval structure
*             result -- (OUT): result interval
*
*  Return Value: Error code
*
*  Errors: none
*
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_repack_i(interval_type interval, interval_type result);

/*
*  Routine:  tadt_swizzle_e
*
*  Description:  Unpacks an event timestamp into the unpacked format.
*
*  Arguments: event -- (IN): event structure
*             result -- (OUT): result event
*
*  Return Value: Error code
*
*  Errors: none
*
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_swizzle_e(packed_event_type event, event_type result);

/*
*  Routine:  tadt_swizzle_fs
*
*  Description:  Unpacks a span timestamp into the unpacked format.
*
*  Arguments: span -- (IN): span structure
*             result -- (OUT): result span
*
*  Return Value: Error code
*
*  Errors: none
*
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_swizzle_fs(packed_span_type span, span_type result);

/*
*  Routine:  tadt_swizzle_i
*
*  Description:  Unpacks an interval timestamp into the unpacked format.
*
*  Arguments: interval -- (IN): interval structure
*             result -- (OUT): result interval
*
*  Return Value: Error code
*
*  Errors: none
*
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_swizzle_i(interval_type interval, interval_type result);

/*
*  Routine:  tadt_create_e 
*
*  Description:  Builds an event timestamp from the given chronon information.
*
*
*  Arguments: granularity -- (IN): granularity
*             size -- (IN): size of the thing
*             granules    -- (IN): number of granules
*             result -- (OUT): result event
*
*  Return Value: Error code
*
*  Errors: none
*
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_create_e(granularity_type granularity, int size, polymorphic_int_type granules, event_type result);


/*  
*  Routine:  tadt_create_i
*
*  Description:  Builds an interval timestamp from the given events.  Assumes
*     storage for the interval has been pre-allocated.
*
*  Arguments: event1 -- (IN): starting event
*             event2 -- (IN): terminating event
*             result -- (OUT): result interval
*
*  Return Value: Error code
*
*  Errors: out_of_range
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_create_i(event_type event1, event_type event2, interval_type result);

/*
*  Routine:  tadt_create_fs
*
*  Description:  Builds a fixed span from the given chronon information.
*
*  Arguments: granularity -- (IN): granularity
*             size -- (IN): size of the span to create
*             granules    -- (IN): number of granules
*             result -- (OUT): result span
*
*  Return Value: Error code
*
*  Errors: none
*
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_create_fs(granularity_type granularity, int sizet, polymorphic_int_type granules, span_type result);


/*  
*  Routine:  tadt_create_vs
*
*  Description:  Builds a variable span from the given information.
*
*  Arguments: vs -- (IN):   vspan structure (contains vspan info)
*             result -- (OUT): resulting variable span
*
*  Return Value: Error code
*
*  Errors: None
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_create_vs(vs_type vs, span_type result);

/*  
*  Routine:  tadt_extract_vs
*
*  Description:  Extract the variable span information from the given vspan.
*
*  Arguments: vs -- (IN):  variable span time-stamp
*             result -- (OUT): resulting variable span information structure
*
*  Return Value: Error code
*
*  Errors: None
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_extract_vs(span_type vs, vs_type result);

/*  
*  Routine:  tadt_is_vs
*
*  Description:  Determine if a span is a variable span.
*
*  Arguments: span -- (IN):  the timestamp to be tested
*
*  Return Value: TRUE if span is a variable span
* 		 FALSE otherwise
*  Errors: None
*  
*  Side Effects: None
*/

bool tadt_is_vs(span_type span);

/*  
*  Routine:  tadt_can_fit_ts
*
*  Description:  Determine if there is enough space allocated to 
*                store a time-stamp.  Time-stamp sizes can be
*                determined using the sizeof function.
*
*  Arguments: ts -- (IN):  the timestamp to be tested 
*             size -- (IN):  how much space is available
*
*  Return Value: TRUE if there is enough space 
* 		 FALSE otherwise
*  Errors: None
*  
*  Side Effects: None
*/

bool tadt_can_fit_ts(timestamp_type ts, int size);

/*  
*  Routine:  tadt_coerce_e
*
*  Description:  Fit an event time-stamp into a certain space, truncating if
*                needed.  An extended resolution event time-stamp must be
*                coerced if it is to fit in the space of a high or 
*                low resolution time-stamp.  If the extended resolution
*                is in the high resolution range, then it is coerced
*                to a high resolution, otherwise it is coerced to a
*                low resolution with truncation flagged.  Time-stamp sizes
*                can be determined using the sizeof function.  Only valid
*                event time-stamp sizes are allowed.  Other sizes will 
*                generate an error.
*
*  Arguments: event -- (IN):  the timestamp to be tested 
*             size -- (IN):  how much space is available
*             result -- (OUT):  the coerced result
*
*  Return Value: Error code
*  
*  Errors: tadt_truncation
*          tadt_invalid_size
*  
*  Side Effects: Assignment to passed result timestamp
*/

tadt_error_type tadt_coerce_e(event_type event, int size, event_type result);

/*  
*  Routine:  tadt_coerce_i
*
*  Description:  Fit an interval time-stamp into a certain space, truncating if
*                needed.  An extended resolution interval time-stamp must be
*                coerced if it is to fit in the space of a high or 
*                low resolution time-stamp.  If the extended resolution
*                is in the high resolution range, then it is coerced
*                to a high resolution, otherwise it is coerced to a
*                low resolution with truncation flagged.  Time-stamp sizes
*                can be determined using the sizeof function.  Only valid
*                interval time-stamp sizes are allowed.  Other sizes will 
*                generate an error.
*
*  Arguments: interval -- (IN):  the timestamp to be tested 
*             size1 -- (IN):  how much space is available for the starting event
*             size2 -- (IN):  how much space is available for the terminating event
*             result -- (OUT):  the coerced result
*
*  Return Value: Error code
*  
*  Errors: tadt_truncation
*          tadt_invalid_size
*  
*  Side Effects: Assignment to passed result timestamp
*/

tadt_error_type tadt_coerce_i(interval_type interval, int size1, int size2, interval_type result);

/*  
*  Routine:  tadt_coerce_fs
*
*  Description:  Fit a fixed span time-stamp into a certain space, truncating 
*                if needed.  An extended resolution span time-stamp must be
*                coerced if it is to fit in the space of a high or 
*                low resolution time-stamp.  If the extended resolution
*                is in the high resolution range, then it is coerced
*                to a high resolution, otherwise it is coerced to a
*                low resolution with truncation flagged.  Time-stamp sizes
*                can be determined using the sizeof function.  Only valid
*                fixed span time-stamp sizes are allowed.  Other sizes will 
*                generate an error.
*
*  Arguments: span -- (IN):  the timestamp to be tested 
*             size -- (IN):  how much space is available
*             result -- (OUT):  the coerced result
*
*  Return Value: Error code
*  
*  Errors: tadt_truncation
*          tadt_invalid_size
*  
*  Side Effects: Assignment to passed result timestamp
*/

tadt_error_type tadt_coerce_fs(span_type span, int size, span_type result);

/*
*  Routine:  tadt_fs_to_seconds
*
*  Description:  Break a fixed span up into its unpacked form (seconds struct)
*
*  Arguments: fs -- (IN):  the fixed span to be converted
*             seconds -- (OUT):  the resulting seconds structure
*
*  Return Value: Error code
*
*  Errors: None
*
*  Side Effects: Assignment to passed seconds structure
*/

error_type tadt_fs_to_seconds(span_type fs, seconds_type seconds);

/*  
*  Routine:  tadt_e_to_seconds
*
*  Description:  Break an event up into its unpacked form (seconds structure)
*
*  Arguments: event -- (IN):  the event to be converted
*             seconds -- (OUT):  the resulting seconds structure
*
*  Return Value: Error code
*  
*  Errors: None
*  
*  Side Effects: Assignment to passed seconds structure
*/

error_type tadt_e_to_seconds(event_type event, seconds_type seconds);

/*  
*  Routine:  tadt_seconds_to_e
*
*  Description:  Convert an unpacked event into its packed form
*
*  Arguments: seconds -- (IN):  the seconds structure to be converted
*             e -- (OUT):  the resulting event
*
*  Return Value: Error code
*  
*  Errors: None
*  
*  Side Effects: Assignment to passed seconds structure
*/

error_type tadt_seconds_to_e(seconds_type seconds, event_type event);

#endif
