/* 
* Routine: tadt_copy_e
*  
* Description:  Copies an event 
*
* Arguments:  ev1 -- (IN): from event
*             ev2 -- (OUT): to event
* 
* Return Value: None 
* 
* Errors: None, we assume that there is enough space to receive the event
*
* Side Effects: Assignment to passed event structure
*/

void tadt_copy_e(event_type ev1, event_type ev2);

/*
* Routine: tadt_copy_i 
*
* Description:  Copies an interval
*
* Arguments:  interval1 -- (IN): from interval
*             interval2 -- (OUT): to interval
*
* Return Value: None
*
* Errors: None
*
* Side Effects: Assignment to passed interval structure
*/

void tadt_copy_i(interval_type interval1, interval_type interval2);

/*
* Routine: tadt_copy_s
*
* Description:  Copies an span
*
* Arguments:  span1 -- (IN): from span
*             span2 -- (OUT): to span
*
* Return Value: None
*
* Errors: None
*
* Side Effects: Assignment to passed span structure
*/

void tadt_copy_s(span_type span1, span_type span2);

/* 
* Routine:  tadt_begin
*  
* Description:  Returns the event delimiting the beginning of the interval
*
* Arguments:  interval -- (IN): the input interval
*             result -- (OUT): result event
* 
* Return Value:  Error code
* 
* Errors: None
*
* Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_begin(interval_type interval, event_type result);

/*
* Routine:  tadt_end
* 
* Description:  Returns the event delimiting the end of the interval
*
* Arguments:  interval -- (IN): the input interval
*             result -- (OUT): result event
*
* Return Value: Error code
* 
* Errors: None
*
* Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_end(interval_type interval, event_type result);

/* 
* Routine:  tadt_interval
*  
* Description:  Constructs an interval from two input events,
*	 	if event1 >= event2 then it flags an error
*
* Arguments:  event1 -- (IN): the beginning of the interval
*	      event2 -- (IN): the end of the interval
*             result -- (OUT): result interval
*
* Return Value: Error code
* 
* Errors: tadt_invalid_interval
*
* Side Effects: assignment to passed result structure
*/

tadt_error_type tadt_interval(event_type event1, event_type event2, interval_type result) ;

/* 
* Routine: tadt_intersect
*  
* Description: Returns the overlap of two intervals, if
*	       they intersect.  Otherwise leave the result
*              unchanged.
*
* Arguments:  i1, i2 -- (IN): intervals for intersection
*             result -- (OUT): result interval if intervals intersect 
*
* Return Value: Error code
* 
* Errors: tadt_invalid_interval
*
* Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_intersect(interval_type i1, interval_type i2, interval_type result);

/* 
* Routine: tadt_span
*  
* Description:  Returns the span of an input interval.  The span is the 
*               inclusive distance between the instants that delimit the 
*               interval
*
* Arguments: interval -- (IN): the input interval
*            result -- (OUT): result span
*
* Return Value: Error code
* 
* Errors: None
*
* Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_span(interval_type i, span_type result);

/* 
*  Routine: tadt_abs_fs
*  
*  Description: Returns the absolute value of a fixed span
*
*  Arguments: span -- (IN): input span
*             result -- (OUT): result positive span
*
*  Return Value:  Error code
* 
*  Errors: None
*
*  Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_abs_fs(span_type span, span_type result);

/* 
*  Routine: tadt_first
*  
*  Description: Returns the earlier of two events
*
*  Arguments: event1, event2 -- (IN): two input events
*             result -- (OUT) earliest event.
*
*  Return Value: Error code
* 
*  Errors: None
*
*  Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_first(event_type event1, event_type event2, event_type result);

/* 
*  Routine: tadt_last
*  
*  Description: Returns the later event of two events
*
*  Arguments: event1, event2 -- (IN): two input events
*             result -- (OUT): latest event.
*
*  Return Value:  Error code
* 
*  Errors: None
*
*  Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_last(event_type event1, event_type event2, event_type result);

/* 
*  Routine: tadt_present_as_of_query
*  
*  Description: Returns the present time for a given query, 
*               the resolution of the passed parameter 
*               is the resolution of the result.  Essentailly
*               this function just caches the value of present
*               so that it will be consistent over the entire 
*               query.  If a new present is desired, the first argument
*               should be TRUE, if the cached value is desired, 
*               then it should be FALSE.
*
*  Arguments: resolution -- (IN): resolution of event
*             result -- (OUT): present event
*
*  Return Value:  Error code
* 
*  Errors: None
*
*  Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_present_as_of_query(bool new, event_type result);

/* 
*  Routine: tadt_present
*  
*  Description: Returns the present time, the resolution of the
*               passed parameter is the resolution of the result.
*
*  Arguments: result -- (OUT): present event
*
*  Return Value:  Error code
* 
*  Errors: None
*
*  Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_present(event_type result);

/* 
*  Routine: tadt_leap_seconds_in
*  
*  Description: Returns the number of leap seconds to a particular
*               time.  The time is given in the seconds structure.
*               Uses binary search to find the time.
*
*  Arguments: seconds -- (IN): seconds structure
*             result -- (OUT): number of seconds
*
*  Return Value:  Error code
* 
*  Errors: None
*
*  Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_leap_seconds_in(seconds_type seconds, int *result);

