/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/newh/tadti/comparison.h
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#ifndef tadti_comparison
#define tadti_comparison

/*  
*  Routine:  tadti_fs_eq_fs
*
*  Description: This routine compares two fixed span values.  It returns
*		the boolean value TRUE if they are equal and FALSE otherwise.
*
*  Arguments: 	gamma, span1, span2 -- (IN) : plausibility and spans to 
*                                             be compared
*
*  Return Value: TRUE if span1 == span2 with plaus. > gamma, FALSE otherwise
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadti_fs_eq_fs(int gamma, span_type span1, span_type span2);

/*  
*  Routine:  tadti_fs_lt_fs
*
*  Description: This routine compares two fixed span values.  It returns
*		the boolean value TRUE if the first span is less than
*		the second span and FALSE otherwise.
*
*  Arguments: 	gamma, span1, span2 -- (IN) : plaus. and spans to be compared
*
*  Return Value: TRUE if span1 < span2 with plaus < gamma, FALSE otherwise
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadti_fs_lt_fs(int gamma, span_type span1, span_type span2);

/*  
*  Routine:  tadti_fs_gt_fs
*
*  Description: This routine compares two fixed span values.  It returns
*		the boolean value TRUE if the first span is greater than
*		the second span and FALSE otherwise.
*
*  Arguments: 	gamma, span1, span2 -- (IN) : plaus. and spans to be compared
*
*  Return Value: TRUE if span1 > span2 with plaus > gamma,FALSE if span1 =<span2
*  
*  Errors: None
*
*  Side Effects: None
*/

bool tadti_fs_gt_fs(int gamma, span_type span1, span_type span2);

/*  
*  Routine:  tadti_e_eq_e
*
*  Description: This routine compares two event values.  It returns
*		the boolean value TRUE if the events are equal and
*		FALSE otherwise.  Two events are defined to be
*		equal if they occur during the same chronon,
*		not if they occur at the same time.     
*		The size of a chronon is resolution dependent.
*
*  Arguments: 	gamma, event1, event2 -- (IN) : plaus. and events to be compared
*
*  Return Value: TRUE if event1 = event2 with plaus. < gamma, 
*                FALSE if event1 <> event2
*  
*  Errors: None
*  
*  Side Effects: This function also checks span values since it can
*                be used for spans.
*/

bool tadti_e_eq_e(int gamma, event_type ev1, event_type ev2);

/*  
*  Routine:  tadti_e_precedes_e
*
*  Description: This routine compares two event values.  It returns
*		the boolean value TRUE if the first event precedes the
*		second event and FALSE otherwise.
*
*  Arguments: 	gamma, event1, event2 -- (IN) : events to be compared
*
*  Return Value: TRUE if event1 < event2 with plaus. < gamma,
*                FALSE if event1 >= event2
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadti_e_precedes_e(int gamma, event_type ev1, event_type ev2);

/*  
*  Routine:  tadti_e_precedes_i
*
*  Description: This routine compares an event and an interval.  It returns
*		the boolean value TRUE if the event precedes the
*		beginning of the interval and FALSE otherwise.
*
*  Arguments: 	gamma, event, interval -- (IN) : plaus., event, 
*                                                and interval to be compared
*
*  Return Value: TRUE if event < begin(interval) with plaus. < gamma,
*                FALSE otherwise
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadti_e_precedes_i(int gamma, event_type event, interval_type interval);

/*  
*  Routine:  tadti_e_overlaps_i
*
*  Description: This routine compares an event and an interval.  It returns
*		the boolean value TRUE if the event overlaps the
*		interval and FALSE otherwise.
*
*  Arguments: 	gamma, event, interval -- (IN) : plaus., event and interval 
*                                                to be compared
*
*  Return Value: TRUE if begin(interval) <= event <= end(interval) with p<gamma
*		 FALSE otherwise
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadti_e_overlaps_i(int gamma, event_type event, interval_type interval);

/*  
*  Routine:  tadti_i_precedes_e
*
*  Description: This routine compares an interval and an event.  It returns
*		the boolean value TRUE if the interval precedes the
*		event and FALSE otherwise.
*
*  Arguments: 	gamma, interval, event -- (IN) : gamma, interval and event 
*                                                to be compared
*
*  Return Value: TRUE if end(interval) < event with plaus. < gamma
*		 FALSE otherwise
*  
*  Errors: None
*
*  Side Effects: None
*/

bool tadti_i_precedes_e(int gamma, interval_type interval, event_type event);

/*  
*  Routine:  tadti_i_precedes_i
*
*  Description: This routine compares two intervals.  It returns
*		the boolean value TRUE if the end of the first interval 
*		precedes the beginning of the second.
*
*  Arguments: 	gamma, interval1, interval2 -- (IN) : plaus and intervals to 
*                                                     be compared
*
*  Return Value: TRUE if end(interval1) < begin(interval2) with plaus. < gamma
*		 FALSE otherwise
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadti_i_precedes_i(int gamma, interval_type interval1, interval_type interval2);

/*  
*  Routine:  tadti_i_eq_i
*
*  Description: This routine compares two intervals.  It returns
*		the boolean value TRUE if the two intervals are
*		equal and FALSE otherwise.
*
*  Arguments: 	gamma, interval1, interval2 -- (IN) : gamma and intervals to 
*                                                     be compared
*
*  Return Value: TRUE if interval1 = interval2 with plaus < gamma
*		 FALSE otherwise
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadti_i_eq_i(int gamma, interval_type interval1, interval_type interval2);

/*  
*  Routine:  tadti_i_meets_i
*
*  Description: This routine compares two intervals.  It returns
*		the boolean value TRUE if the end of the first interval
*		is the beginning of the second and FALSE otherwise.
*
*  Arguments: 	gamma, interval1, interval2 -- (IN) : intervals to be compared
*
*  Return Value: TRUE if end(interval1) = begin(interval2) with plaus < gamma
*                FALSE otherwise
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadti_i_meets_i(int gamma, interval_type interval1, interval_type interval2);

/*  
*  Routine:  tadti_i_overlaps_i
*
*  Description: This routine compares two intervals.  It returns
*		the boolean value TRUE if the intervals share at least
*		one chronon in common and FALSE otherwise.
*
*  Arguments: 	gamma, interval1, interval2 -- (IN) : intervals to be compared
*
*  Return Value: TRUE if interval1 overlaps interval2 with plaus. < gamma
*                FALSE otherwise
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadti_i_overlaps_i(int gamma, interval_type interval1, interval_type interval2);

/*  
*  Routine:  tadti_i_contains_i
*
*  Description: This routine compares two intervals.  It returns
*		the boolean value TRUE if the first interval contains
*		the second interval and FALSE otherwise.
*
*  Arguments: 	gamma, interval1, interval2 -- (IN) : intervals to be compared
*
*  Return Value: TRUE if interval1 contains interval2 with plaus < gamma
*                FALSE otherwise
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadti_i_contains_i(int gamma, interval_type i1, interval_type i2);
#endif
