/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/newh/ucs/arithmetic.h
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#ifndef ucs_arithmetic
#define ucs_arithmetic

/* arithmetic.h
*
*  This module implements UCS routines supporting 
*  span arithmetic operations.
*  Routines in this module are called by the query
*  processing system when computing arithmetic operations
*  involving spans.
*
*/

/*  
*  Routine:  ucs_neg_s
*
*  Description: Return a negative span whose duration is the
*		same as the input span.
*
*  Arguments: 	spanin -- (IN) : span to be converted
*               result-- (OUT): result negative span
*
*  Return Value: Error code
*
*  Errors: semantic_error
*  
*  Side Effects: Overwrites result
*/

error_type ucs_neg_s(span_type spanin, span_type result);
/*  
*  Routine:  ucs_s_add_s
*
*  Description: Compute the sum of two spans
*
*  Arguments: 	span1, span2 --  (IN) : spans to be added
*               result       --  (OUT): the sum of the arguments
*
*  Return Value: Error code
*
*  Errors: out_of_range
*
*  Side Effects: Overwrites result
*/

error_type ucs_s_add_s(span_type span1, span_type span2, span_type result);

/*  
*  Routine:  ucs_s_minus_s
*
*  Description: Compute the difference of two spans
*
*  Arguments: 	span1 --   (IN) : minuend (span to be subtracted from)
*		span2 --   (IN) : subtrahend
*               result--   (OUT): the result of "span1 - span2"
*
*  Return Value: Error code 
*
*  Errors: out_of_range
*    
*  Side Effects: Overwrites result
*/

error_type ucs_s_minus_s(span_type span1, span_type span2, span_type result);

/*  
*  Routine:  ucs_e_add_s
*
*  Description: Displace an event by adding a span
*
*  Arguments: event_in --      (IN) : the event time
*	      span     --      (IN) : the increment period
*             result --     (OUT) : the new event time after displacement
*
*  Return Value: Error code
*
*  Errors: out_of_range
*    
*  Side Effects: Overwrites result
*/

error_type ucs_e_add_s(event_type event_in, span_type span, event_type result);

/*  
*  Routine:  ucs_e_minus_s
*
*  Description: Displace an event by subtracting a duration of time (span)
*
*  Arguments: event_in  --      (IN) : the event time
*	      span      --      (IN) : the decrement period
*             result    --     (OUT) : the new event time after decrement
*
*  Return Value: Error code
*
*  Errors: out_of_range
*    
*  Side Effects: Overwrites result
*/

error_type ucs_e_minus_s(event_type event_in, span_type span, event_type result);

/*  
*  Routine:  ucs_s_times_n
*
*  Description: This function multiplies a span by a numeric value
*
*  Arguments:   span_in   --    (IN) : the input span
*		n         --    (IN) : number of times
*               result    --   (OUT) : the result of (span x n)
*
*  Return Value: Error code
*
*  Errors: out_of_range
*    
*  Side Effects: Overwrites result
*/

error_type ucs_s_times_n(span_type span_in, int n, span_type result);

/*  
*  Routine:  ucs_s_div_n
*
*  Description: Return a span equal to the division of the input span by
*		a numeric value
*
*  Arguments: span_in  --     (IN) :the input span
*	      n        --     (IN) : number of division
*             result   --     (OUT): the result of (span / n)
*
*  Return Value: Error code
*
*  Errors: divide_by_zero
*    
*  Side Effects: Overwrites result
*/

error_type ucs_s_div_n(span_type span_in, double n, span_type result);

/*  
*  Routine:  ucs_s_div_s
*
*  Description: Divide one span by another
*
*  Arguments:   span1    --     (IN): the dividend
*		span2    --     (IN): the divisor
*               dividend --    (OUT): the result of (s/s)
*
*  Return Value: Error code
*
*  Errors: divide_by_zero
*    
*  Side Effects: Overwrites dividend
*/

error_type ucs_s_div_s(span_type span1, span_type span2, double *dividend);

/*  
*  Routine:  ucs_i_add_s
*
*  Description: Displace an interval by adding a duration (span) of time
*
*  Arguments: interval_in  --      (IN) : the interval to be displaced
*	      span         --      (IN) : the span of displacement
*             result       --      (OUT): interval displace by span
*
*  Return Value: Error code
*
*  Errors: out_of_range
*    
*  Side Effects: Overwrites result
*/

error_type ucs_i_add_s (interval_type interval_in, span_type span, interval_type result);

/*  
*  Routine:  ucs_i_minus_s
*
*  Description: Displace an interval by subtracting a duration (span) of time
*
*  Arguments: interval_in   --   (IN) : the interval to be displaced
*	      span          --   (IN) : the span of displacement
*             result        --  (OUT) : interval displaced by span
*
*  Return Value: Error code
*
*  Errors: out_of_range
*    
*  Side Effects: Overwrites result
*/

error_type ucs_i_minus_s (interval_type interval_in, span_type span, interval_type result);
#endif
