/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/tadt/tadt_builtin.c
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include "system_wide.h"
#include "tadt.h"

extern size_type sizes[MAX_RESOLUTIONS_OF_EVENTS][MAX_PRECISION];
extern context_type context;

/* 
* Routine: tadt_copy_e
*  
* Description:  Copies an event 
*
* Arguments:  ev1 -- (IN): from event
*             ev2 -- (OUT): to event
* 
* Return Value: None 
* 
* Errors: None, we assume that there is enough space to receive the event
*
* Side Effects: Assignment to passed event structure
*/

void tadt_copy_e(event_type ev1, event_type ev2)
{
  int i;

#define WORD_COPY(y) \
    for (i = 0; i < sizeof(y)/4; i++) ev2->word[i] = ev1->word[i]
  
  /* If same then do nothing */
  if (ev1 == ev2) return;

  /* Do a word by word copy */
  /* All events are at least two words */
  switch (ev1->generic.ts_type) {
    case determinate:
       WORD_COPY(determinate_type); 
       break;
    case upack:
       WORD_COPY(upack_determinate_type); 
       break;
    case special:
       WORD_COPY(special_type); break;
    case nowrelative:
       WORD_COPY(nowrelative_type); break;

#ifdef INDETERMINATE
    case indeterminate:
       WORD_COPY(general96_nonstandard_type);  /*This is the biggest ind type*/
       break;
    case upack_ind:
       WORD_COPY(upack_indeterminate_type); 
       break;
#endif

    default: 
        internal_error("%s %d", "tadt_copy_e: bad ts_type ",ev1->generic.ts_type);
    }
}

/*
* Routine: tadt_copy_i 
*
* Description:  Copies an interval
*
* Arguments:  interval1 -- (IN): from interval
*             interval2 -- (OUT): to interval
*
* Return Value: None
*
* Errors: None
*
* Side Effects: Assignment to passed interval structure
*/

void tadt_copy_i(interval_type interval1, interval_type interval2) {

  /* If same then do nothing */
  if (interval1 == interval2) return;

  tadt_copy_e(interval1->start, interval2->start);
  tadt_copy_e(interval1->terminate, interval2->terminate);
}

/*
* Routine: tadt_copy_s
*
* Description:  Copies an span
*
* Arguments:  span1 -- (IN): from span
*             span2 -- (OUT): to span
*
* Return Value: None
*
* Errors: None
*
* Side Effects: Assignment to passed span structure
*/

void tadt_copy_s(span_type span1, span_type span2) {
   int i;

  /* If same then do nothing */
  if (span1 == span2) return;

   if (span1->generic.ts_type == special && span1->special.vspan) {
      for (i = 0; i < sizeof(vspan_type)/4; i++) span2->word[i] = span1->word[i];
      } 
   else 
     /* Exploit fixed span event similarity */
     tadt_copy_e((event_type) span1, (event_type) span2);
}

/* 
* Routine:  tadt_begin
*  
* Description:  Returns the event delimiting the beginning of the interval
*
* Arguments:  interval -- (IN): the input interval
*             result -- (OUT): result event
* 
* Return Value:  Error code
* 
* Errors: None
*
* Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_begin(interval_type interval, event_type result)
{
  tadt_copy_e(interval->start,result);
  return tadt_ok;
}

/*
* Routine:  tadt_end
* 
* Description:  Returns the event delimiting the end of the interval
*
* Arguments:  interval -- (IN): the input interval
*             result -- (OUT): result event
*
* Return Value: Error code
* 
* Errors: None
*
* Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_end(interval_type interval, event_type result)
{
  tadt_copy_e(interval->terminate,result);
  return tadt_ok;
}

/* 
* Routine:  tadt_interval
*  
* Description:  Constructs an interval from two input events,
*	 	if event1 >= event2 then it flags an error
*
* Arguments:  event1 -- (IN): the beginning of the interval
*	      event2 -- (IN): the end of the interval
*             result -- (OUT): result interval
*
* Return Value: Error code
* 
* Errors: tadt_invalid_interval
*
* Side Effects: assignment to passed result structure
*/

tadt_error_type tadt_interval(event_type event1, event_type event2, interval_type result) 
{
  tadt_copy_e(event1,result->start);
  tadt_copy_e(event2,result->terminate);
  return tadt_ok;
}


/* 
* Routine: tadt_intersect
*  
* Description: Returns the overlap of two intervals, if
*	       they intersect.  Otherwise leave the result
*              unchanged.
*
* Arguments:  i1, i2 -- (IN): intervals for intersection
*             result -- (OUT): result interval if intervals intersect 
*
* Return Value: Error code
* 
* Errors: tadt_invalid_interval
*
* Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_intersect(interval_type i1, interval_type i2, interval_type result)
{
  if (tadt_i_overlaps_i(i1,i2)) {
    tadt_first(i1->terminate,i2->terminate,result->terminate);
    tadt_last(i1->start,i2->start,result->start);
    return tadt_ok;
    }
  return tadt_invalid_interval;
}

/* 
* Routine: tadt_span
*  
* Description:  Returns the span of an input interval, a positive span
*               Span is the inclusive distance between the instants 
*               that delimit the interval
*
* Arguments: interval -- (IN): the input interval
*            result -- (OUT): result span
*
* Return Value: Error code
* 
* Errors: None
*
* Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_span(interval_type i, span_type result)
{
  span_space_type temp, temp1;
  tadt_error_type rc;
  size_type size;

  temp.generic.ts_type = upack;
  size = sizes[upack][context->precision.precision];
  switch (size) {
    case SIZE32: LOAD32(1,temp.upack32.granules); break;
    case SIZE64: LOAD64(1,temp.upack64.granules); break;
    case SIZE96: LOAD96(1,temp.upack96.granules); break;
    default: internal_error("%s", "in tadt_span size table corrupt" );
    }
  if ((rc = tadt_e_minus_e(i->terminate,i->start,&temp1)) == tadt_ok) {
    /* Off by one, so add it back */
    rc = tadt_fs_add_fs(&temp1, &temp, result);
    }
  return rc;
}

/* 
*  Routine: tadt_abs_fs
*  
*  Description: Returns the absolute value of a fixed span
*
*  Arguments: span -- (IN): input span
*             result -- (OUT): result positive span
*
*  Return Value:  Error code
* 
*  Errors: None
*
*  Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_abs_fs(span_type span, span_type result)
{
  extern span_type tadt_zero_span;

  /* Exploit span, event similarity */
  if (tadt_fs_lt_fs(span, tadt_zero_span)) tadt_neg_fs(span, result);
  else tadt_copy_s(span, result);
  return tadt_ok;
}

/* 
*  Routine: tadt_first
*  
*  Description: Returns the earlier of two events
*
*  Arguments: event1, event2 -- (IN): two input events
*             result -- (OUT) earliest event.
*
*  Return Value: Error code
* 
*  Errors: None
*
*  Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_first(event_type event1, event_type event2, event_type result)
{
  if (tadt_e_precedes_e(event1,event2)) 
     tadt_copy_e(event1,result);
  else tadt_copy_e(event2,result);
  return tadt_ok;
}

/* 
*  Routine: tadt_last
*  
*  Description: Returns the later event of two events
*
*  Arguments: event1, event2 -- (IN): two input events
*             result -- (OUT): latest event.
*
*  Return Value:  Error code
* 
*  Errors: None
*
*  Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_last(event_type event1, event_type event2, event_type result)
{
  if (tadt_e_precedes_e(event1,event2)) 
     tadt_copy_e(event2,result);
  else tadt_copy_e(event1,result);
  return tadt_ok;
}

/* 
*  Routine: tadt_present_as_of_query
*  
*  Description: Returns the present time for a given query, 
*               the resolution of the passed parameter 
*               is the resolution of the result.  Essentailly
*               this function just caches the value of present
*               so that it will be consistent over the entire 
*               query.  If a new present is desired, the first argument
*               should be TRUE, if the cached value is desired, 
*               then it should be FALSE.
*
*  Arguments: resolution -- (IN): resolution of event
*             result -- (OUT): present event
*
*  Return Value:  Error code
* 
*  Errors: None
*
*  Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_present_as_of_query(bool new, event_type result)
{
  static event_space_type cached_present;

  if (new) tadt_present(&cached_present);
  tadt_copy_e(&cached_present, result);
  return ok;
}

/* 
*  Routine: tadt_present
*  
*  Description: Returns the present time, the resolution of the
*               passed parameter is the resolution of the result.
*
*  Arguments: result -- (OUT): present event
*
*  Return Value:  Error code
* 
*  Errors: None
*
*  Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_present(event_type result)
{
  struct timeval tp;
  int gettimeofday();

  gettimeofday(&tp, NULL);
  /* Result in scale of seconds at a size of 32 */
  result->upack32.granules = tp.tv_sec;
  result->upack.size = SIZE32;
  result->upack.ts_type = upack;
  result->upack.modified = TRUE;
  return tadt_ok;

}

/* 
*  Routine: tadt_leap_seconds_in
*  
*  Description: Returns the number of leap seconds to a particular
*               time.  The time is given in the seconds structure.
*               Uses binary search to find the time.
*
*  Arguments: seconds -- (IN): seconds structure
*             result -- (OUT): number of seconds
*
*  Return Value:  Error code
* 
*  Errors: None
*
*  Side Effects: Assignment to passed result structure
*/

tadt_error_type tadt_leap_seconds_in(seconds_type seconds, int *result)
{
  /*
  extern unsigned long leap_second_probe_table[], leap_second_table[];
  int left, right, range, test, mid;
  unsigned long probe;
  */
  
  /* No leap seconds before the origin */
  /* HACK OUT FOR NOW 
  switch (seconds->size) {
    case SIZE32: 
      seconds->seconds32 
      break
    case SIZE64: result
    case SIZE96: result

  if (seconds->seconds < leap_second_probe_table[0]) {
     *result = 0;
     return tadt_ok;
     }
  END OF HACKING OUT */
  /* This is a quick hack!!! I am assuming that we will be outdated in
     100 years and this section will no longer be here */
  /* No leap seconds after 2031, so can ignore hi order word*/
  /* HACK OUT FOR NOW 
  if (seconds->seconds >> 32) {
     *result = MAX_LEAP_SECOND;
     return tadt_ok;
     }

  END OF HACKING OUT */
  /* Binary search to find the leap second just below the probe */
  /* HACK OUT FOR NOW 
  probe = seconds->seconds;
  left = 0;
  right = MAX_LEAP_SECOND;
  for (range = right - left; range > 1; range = right-left) {
    mid  = (range >> 1) + left;
    test = leap_second_probe_table[mid];
    if (test > probe) {
      right = mid;
      }
    else {
      left = mid;
      }
    } 

  END OF HACKING OUT */
  /* I tested and left is the one we want */
  /*
  *result = leap_second_table[left];
  */
  return tadt_ok;
}

