/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/tadt/tadt_comparison.c
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#include <stdlib.h>
#include "system_wide.h"
#include "tadt.h"

/* These macros are efficient interval slicers, We do NOT use tadt_begin,
   tadt_first, etc. for efficiency reasons */

#define BEGIN_OF(x) x->start
#define END_OF(x) x->terminate
#define LAST_OF(x,y) (tadt_e_precedes_e(x,y)? y : x)
#define FIRST_OF(x,y) (tadt_e_precedes_e(x,y)? x : y)

/*  
*  Routine:  tadt_fs_eq_fs
*
*  Description: This routine compares two fixed span values.  It returns
*		the boolean value TRUE if they are equal and FALSE otherwise.
*
*  Arguments: 	span1, span2 -- (IN) : spans to be compared
*
*  Return Value: TRUE if span1 == span2, FALSE otherwise
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadt_fs_eq_fs(span_type span1, span_type span2)
{
  /* Exploit the span/event similarity */
  return tadt_e_eq_e((event_type)span1, (event_type)span2);
}


/*  
*  Routine:  tadt_fs_lt_fs
*
*  Description: This routine compares two fixed span values.  It returns
*		the boolean value TRUE if the first span is less than
*		the second span and FALSE otherwise.
*
*  Arguments: 	span1, span2 -- (IN) : spans to be compared
*
*  Return Value: TRUE if span1 < span2, FALSE otherwise
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadt_fs_lt_fs(span_type span1, span_type span2)
{
  unsigned short flags;

  LAZY_UNPACK_SPAN(span1);

  flags = (span1->generic.ts_type << 4) + span2->generic.ts_type;
  switch (flags) {

    case upack*16 + upack: /* upack and upack*/
      switch (span1->upack.size) {
        case SIZE32: return LT32(span1->upack32.granules, span2->upack32.granules);
        case SIZE64: return LT64(span1->upack64.granules, span2->upack64.granules);
        case SIZE96: return LT96(span1->upack96.granules, span2->upack96.granules);
        }

    case upack*16 + special: /* upack and special */
      switch (span2->special.which) {
	case negative_all_of_time: return FALSE;
	case all_of_time: return TRUE;
	case undefined: return FALSE; /* internal_error */
        default: internal_error("%s", "tadt_fs_lt_fs: bad special type");
	}

    case special*16 + upack: /* special and upack */
      switch (span1->special.which) {
	case negative_all_of_time: return TRUE;
	case all_of_time: return FALSE;
	case undefined: return FALSE; /* internal_error */
        default: internal_error("%s", "tadt_fs_lt_fs: bad special type");
	}

    case special*16 + special: /* special and special */
      switch (span1->special.which) {
	case negative_all_of_time: 
	  switch (span2->special.which) {
	    case all_of_time: return TRUE;
	    case negative_all_of_time: return FALSE;
	    case undefined: return FALSE; /* internal_error */
	    default: internal_error("%s", "tadt_fs_lt_fs: bad special type");
	    }
	case all_of_time: 
	  switch (span2->special.which) {
	    case all_of_time: return FALSE;
	    case negative_all_of_time: return FALSE;
	    case undefined: return FALSE; /* internal_error */
	    default: internal_error("%s", "tadt_fs_lt_fs: bad special type");
	    }
	case undefined: return FALSE; /* internal_error */
	default: internal_error("%s", "tadt_fs_lt_fs: bad special type");
        }
      
    default:
      internal_error("%s", "tadt_fs_lt_fs: BAD Flags");

  }

  internal_error("%s", "tadt_fs_lt_fs: how did I get here?");
  return FALSE;
}

/*  
*  Routine:  tadt_fs_gt_fs
*
*  Description: This routine compares two fixed span values.  It returns
*		the boolean value TRUE if the first span is greater than
*		the second span and FALSE otherwise.
*
*  Arguments: 	span1, span2 -- (IN) : spans to be compared
*
*  Return Value: TRUE if span1 > span2, FALSE if span1 =< span2
*  
*  Errors: None
*
*  Side Effects: None
*/

bool tadt_fs_gt_fs(span_type span1, span_type span2)
{
  unsigned short flags;

  LAZY_UNPACK_SPAN(span1);
  LAZY_UNPACK_SPAN(span2);

  flags = (span1->generic.ts_type << 4) + span2->generic.ts_type;
  switch (flags) {

    case upack*16 + upack: /* upack and upack*/
      switch (span1->upack.size) {
        case SIZE32: return GT32(span1->upack32.granules, span2->upack32.granules);
        case SIZE64: return GT64(span1->upack64.granules, span2->upack64.granules);
        case SIZE96: return GT96(span1->upack96.granules, span2->upack96.granules);
        }

    case upack*16 + special: /* upack and special */
      switch (span2->special.which) {
	case negative_all_of_time: return TRUE;
	case all_of_time: return FALSE;
	case undefined: return FALSE; /* internal_error */
        default: internal_error("%s", "tadt_fs_lt_fs: bad special type");
	}

    case special*16 + upack: /* special and upack */
      switch (span1->special.which) {
	case negative_all_of_time: return FALSE;
	case all_of_time: return TRUE;
	case undefined: return FALSE; /* internal_error */
        default: internal_error("%s", "tadt_fs_lt_fs: bad special type");
	}

    case special*16 + special: /* special and special */
      switch (span1->special.which) {
	case negative_all_of_time: 
	  switch (span2->special.which) {
	    case all_of_time: return FALSE;
	    case negative_all_of_time: return FALSE;
	    case undefined: return FALSE; /* internal_error */
	    default: internal_error("%s", "tadt_fs_lt_fs: bad special type");
	    }
	case all_of_time: 
	  switch (span2->special.which) {
	    case all_of_time: return FALSE;
	    case negative_all_of_time: return TRUE;
	    case undefined: return FALSE; /* internal_error */
	    default: internal_error("%s", "tadt_fs_lt_fs: bad special type");
	    }
	case undefined: return FALSE; /* internal_error */
	default: internal_error("%s", "tadt_fs_lt_fs: bad special type");
        }
      
    default:
      internal_error("%s", "tadt_fs_lt_fs: BAD Flags");

  }

  internal_error("%s", "tadt_fs_lt_fs: how did I get here?");
  return FALSE;
}

/*  
*  Routine:  tadt_e_eq_e
*
*  Description: This routine compares two event values.  It returns
*		the boolean value TRUE if the events are equal and
*		FALSE otherwise.  Two events are defined to be
*		equal if they occur at the same granule,
*		not if they occur at the same *instant*.     
*
*  Arguments: 	event1, event2 -- (IN) : events to be compared
*
*  Return Value: TRUE if event1 = event2, FALSE if event1 <> event2
*  
*  Errors: None
*  
*  Side Effects: This function also checks span values since it can
*                be used for spans.
*/

bool tadt_e_eq_e(event_type ev1, event_type ev2)
{
  unsigned short flags;

  LAZY_UNPACK_EVENT(ev1);
  LAZY_UNPACK_EVENT(ev2);

  flags = (ev1->generic.ts_type << 4) + ev2->generic.ts_type;
  switch (flags) {

    case upack*16 + upack: /* upack and upack*/
      switch (ev1->upack.size) {
        case SIZE32: return EQ32(ev1->upack32.granules, ev2->upack32.granules);
        case SIZE64: return EQ64(ev1->upack64.granules, ev2->upack64.granules);
        case SIZE96: return EQ96(ev1->upack96.granules, ev2->upack96.granules);
        }

    case upack*16 + special: /* upack and special */
      return FALSE;

    case special*16 + upack: /* special and upack */
      return FALSE;

    case special*16 + special: /* special and special */
      if (ev1->special.which == undefined) return FALSE;
      if (ev2->special.which == undefined) return FALSE;
      return (ev1->special.which == ev2->special.which);

    default:
      internal_error("%s", "tadt_e_eq_e: BAD Flags");

  }

  internal_error("%s", "tadt_e_eq_e: how did I get here?");
  return FALSE;
}


/*  
*  Routine:  tadt_e_precedes_e
*
*  Description: This routine compares two event values.  It returns
*		the boolean value TRUE if the first event precedes the
*		second event and FALSE otherwise.
*
*  Arguments: 	event1, event2 -- (IN) : events to be compared
*
*  Return Value: TRUE if event1 < event2, FALSE if event1 >= event2
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadt_e_precedes_e(event_type ev1, event_type ev2)
{
  return LT32(ev1->upack32.granules, ev2->upack32.granules);

    switch (ev1->upack.size) {
      case SIZE32: return LT32(ev1->upack32.granules, ev2->upack32.granules);
      case SIZE64: return LT64(ev1->upack64.granules, ev2->upack64.granules);
      case SIZE96: return LT96(ev1->upack96.granules, ev2->upack96.granules);
      }

  /*
  flags = (ev1->generic.ts_type << 4) + ev2->generic.ts_type;
  */
  switch (ev1->generic.ts_type) {
    case upack:
      switch (ev2->generic.ts_type) {
        case upack: /* upack and upack*/
          switch (ev1->upack.size) {
            case SIZE32: return LT32(ev1->upack32.granules, ev2->upack32.granules);
            case SIZE64: return LT64(ev1->upack64.granules, ev2->upack64.granules);
            case SIZE96: return LT96(ev1->upack96.granules, ev2->upack96.granules);
            }

        case special: /* upack and special */
          switch (ev2->special.which) {
	    case negative_all_of_time: return FALSE;
	    case all_of_time: return TRUE;
	    case beginning: return FALSE;
	    case forever:   return TRUE;
	    case undefined: return FALSE; /* internal_error */
            default: internal_error("%s","tadt_e_precedes_e: bad special type");
	    }
        }

    case special:
      switch (ev2->generic.ts_type) {
        case upack: /* special and upack */
          switch (ev1->special.which) {
	    case negative_all_of_time: return TRUE;
	    case all_of_time: return FALSE;
	    case beginning: return TRUE;
	    case forever:   return FALSE;
	    case undefined: return FALSE; /* internal_error */
            default: internal_error("%s","tadt_e_precedes_e: bad special type");
	    }
        case special: /* special and special */
          switch (ev1->special.which) {
	    case negative_all_of_time: return TRUE;
	    case all_of_time: 
	      switch (ev2->special.which) {
	        case all_of_time: return TRUE;
	        case negative_all_of_time: return FALSE;
	        case undefined: return FALSE; /* internal_error */
	        default: internal_error("%s", "tadt_e_precedes_e: bad special type");
	        }
  	    case beginning: return TRUE;
	    case forever:   
	      switch (ev2->special.which) {
	        case forever: return TRUE;
	        case beginning: return FALSE;
	        case undefined: return FALSE; /* internal_error */
	        default: internal_error("%s", "tadt_e_precedes_e: bad special type");
	        }
	    case undefined: return FALSE; /* internal_error */
	    default: internal_error("%s", "tadt_e_precedes_e: bad special type");
	  }
        }
      
    default:
      internal_error("%s", "tadt_e_precedes_e: BAD Flags");

  }

  internal_error("%s", "tadt_e_precedes_e: how did I get here?");
  return FALSE;
}

/*  
*  Routine:  tadt_e_precedes_i
*
*  Description: This routine compares an event and an interval.  It returns
*		the boolean value TRUE if the event precedes the
*		beginning of the interval and FALSE otherwise.
*
*  Arguments: 	event, interval -- (IN) : event and interval to be compared
*
*  Return Value: TRUE if event < begin(interval),
*                FALSE if event >= begin(interval)
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadt_e_precedes_i(event_type event, interval_type interval)
{
  return tadt_e_precedes_e(event,BEGIN_OF(interval));
}

/*  
*  Routine:  tadt_e_overlaps_i
*
*  Description: This routine compares an event and an interval.  It returns
*		the boolean value TRUE if the event overlaps the
*		interval and FALSE otherwise.
*
*  Arguments: 	event, interval -- (IN) : event and interval to be compared
*
*  Return Value: TRUE if begin(interval) <= event <= end(interval)
*		 FALSE if event < begin(interval) or event > end(interval)
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadt_e_overlaps_i(event_type event, interval_type interval)
{
  if (tadt_e_precedes_e(event, BEGIN_OF(interval))) return FALSE;
  if (tadt_e_precedes_e(END_OF(interval),event)) return FALSE;
  return TRUE;
}

/*  
*  Routine:  tadt_i_precedes_e
*
*  Description: This routine compares an interval and an event.  It returns
*		the boolean value TRUE if the interval precedes the
*		event and FALSE otherwise.
*
*  Arguments: 	interval, event -- (IN) : interval and event to be compared
*
*  Return Value: TRUE if end(interval) < event
*		 FALSE if event <= end(interval)
*  
*  Errors: None
*
*  Side Effects: None
*/

bool tadt_i_precedes_e(interval_type interval, event_type event)
{
  return tadt_e_precedes_e(END_OF(interval),event);
}

/*  
*  Routine:  tadt_i_precedes_i
*
*  Description: This routine compares two intervals.  It returns
*		the boolean value TRUE if the end of the first interval 
*		precedes the beginning of the second.
*
*  Arguments: 	interval1, interval2 -- (IN) : intervals to be compared
*
*  Return Value: TRUE if end(interval1) < begin(interval2)
*		 FALSE if end(interval1) >= begin(interval2)
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadt_i_precedes_i(interval_type interval1, interval_type interval2)
{
  return tadt_e_precedes_e(END_OF(interval1),BEGIN_OF(interval2));
}

/*  
*  Routine:  tadt_i_eq_i
*
*  Description: This routine compares two intervals.  It returns
*		the boolean value TRUE if the two intervals are
*		equal and FALSE otherwise.
*
*  Arguments: 	interval1, interval2 -- (IN) : intervals to be compared
*
*  Return Value: bool TRUE if interval1 = interval2
*		 bool FALSE if interval1 <> interval2
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadt_i_eq_i(interval_type interval1, interval_type interval2)
{
  if (!tadt_e_eq_e(BEGIN_OF(interval1),BEGIN_OF(interval2))) return FALSE;
  if (!tadt_e_eq_e(END_OF(interval1),END_OF(interval2))) return FALSE;
  return TRUE;
}


/*  
*  Routine:  tadt_i_meets_i
*
*  Description: This routine compares two intervals.  It returns
*		the boolean value TRUE if the end of the first interval
*		is the beginning of the second and FALSE otherwise.
*
*  Arguments: 	interval1, interval2 -- (IN) : intervals to be compared
*
*  Return Value: TRUE if end(interval1) = begin(interval2)
*                FALSE if end(interval1) <> begin(interval2)
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadt_i_meets_i(interval_type interval1, interval_type interval2)
{
  return tadt_e_eq_e(END_OF(interval1),BEGIN_OF(interval2));
}

/*  
*  Routine:  tadt_i_overlaps_i
*
*  Description: This routine compares two intervals.  It returns
*		the boolean value TRUE if the intervals share at least
*		one chronon in common and FALSE otherwise.
*
*  Arguments: 	interval1, interval2 -- (IN) : intervals to be compared
*
*  Return Value: TRUE if interval1 overlaps interval2
*                FALSE if interval1 doesn't overlap interval2
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadt_i_overlaps_i(interval_type interval1, interval_type interval2)
{
  if (tadt_e_precedes_e(FIRST_OF(END_OF(interval1),END_OF(interval2)),
		        LAST_OF(BEGIN_OF(interval1),BEGIN_OF(interval2))
		       )) return FALSE;
  return TRUE;
}

/*  
*  Routine:  tadt_i_contains_i
*
*  Description: This routine compares two intervals.  It returns
*		the boolean value TRUE if the first interval contains
*		the second interval and FALSE otherwise.
*
*  Arguments: 	interval1, interval2 -- (IN) : intervals to be compared
*
*  Return Value: TRUE if interval1 contains interval2
*                FALSE if interval1 doesn't contain interval2
*  
*  Errors: None
*  
*  Side Effects: None
*/

bool tadt_i_contains_i(interval_type i1, interval_type i2)
{
  if (tadt_e_precedes_e(BEGIN_OF(i2), BEGIN_OF(i1))) return FALSE;
  if (tadt_e_precedes_e(END_OF(i1), END_OF(i2))) return FALSE;
  return TRUE;
}
