/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/tadt/tadt_timestamp.c
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#include <stdlib.h>
#include "system_wide.h"
#include "tadt.h"

extern size_type sizes[MAX_RESOLUTIONS_OF_EVENTS][MAX_PRECISION];
extern context_type context;

#ifdef INDETERMINATE
extern dist_type distributions[MAX_DISTRIBUTION];
#endif

/*  
*  Routine:  tadt_repack_e
*
*  Description:  Repacks an event timestamp from the unpacked format.
*                We assume enough space has been allocated.  Always
*                repacks to extended resolution.  Use coercing to 
*                fit into a smaller space.
*
*  Arguments: event -- (IN): event structure
*             result -- (OUT): result event
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_repack_e(event_type event, event_type result)
{
  switch (event->generic.ts_type) {
    case determinate:
#ifdef NOWRELATIVE
    case nowrelative:
#endif
#ifdef INDETERMINATE
    case indeterminate:
#endif
      internal_error("%s", "tadt_repack_e: attempted to repack packed event");

    case special: /* do not repack, it is never unpacked */
      break;

    case upack:
      /* Implicit coercion? */
      result->generic.ts_type = event->upack.base_ts_type;
      break;

    default:
      internal_error("%s", "tadt_repack_e: bad passed ts_type");

    }

  return ok;
}

/*
*  Routine:  tadt_repack_fs
*
*  Description:  Repacks a span timestamp from the unpacked format.
*                We assume enough space has been allocated.  Always
*                repacks to extended resolution.  Use coercing to
*                fit into a smaller space.
*
*  Arguments: span -- (IN): span structure
*             result -- (OUT): result span
*
*  Return Value: Error code
*
*  Errors: none
*
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_repack_fs(span_type span, span_type result) {
   /* exploit event, fixed span similarity */
   tadt_repack_e((event_type) span, (event_type) result);
   return ok;
}

/*
*  Routine:  tadt_repack_i
*
*  Description:  Repacks an interval timestamp from the unpacked format.
*                We assume enough space has been allocated.  Always
*                repacks to extended resolution.  Use coercing to
*                fit into a smaller space.
*
*  Arguments: interval -- (IN): interval structure
*             result -- (OUT): result interval
*
*  Return Value: Error code
*
*  Errors: none
*
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_repack_i(interval_type interval, interval_type result) {
   tadt_repack_e(interval->start, result->start);
   tadt_repack_e(interval->terminate, result->terminate);
   return ok;
}

#ifdef INDETERMINATE

/* This is the chunk table */
extern span_space_type chunk32_table[HOW_MANY_CHUNKS][MAX_CHUNK];
extern span_space_type chunk64_table[HOW_MANY_CHUNKS][MAX_CHUNK];
extern span_space_type chunk96_table[HOW_MANY_CHUNKS][MAX_CHUNK];

/* An easy way to specify a single chunk */
#define CHUNK32(field,var) \
   (span_type) &chunk32_table[var->field.chunks][var->field.chunksize]

#define CHUNK64(field,var) \
   (span_type) &chunk64_table[var->field.chunks][var->field.chunksize]

#define CHUNK96(field,var) \
   (span_type) &chunk96_table[var->field.chunks][var->field.chunksize]

/*  
*  Routine:  tadt_unpack_chunked
*
*  Description:  Unpacks a chunked timestamp with a standard distribution.
*
*  Arguments: event -- (IN): event structure
*             result -- (OUT): result event
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

static error_type tadt_unpack_chunked(event_type event, event_type result, precision_type precision)
{
  long temp;
  size_type size;
  int32 temp32;
  int64 temp64;
  int96 temp96;
  event_type result_lower, result_upper;

  /* PRECISION_TO_SIZE(precision,size); */
  size = sizes[chunked][precision->precision];

  result->generic.ts_type = upack;
  result->upack_ind.indeterminate = TRUE;
  result->upack_ind.base_ts_type = indeterminate;
  result->upack_ind.modified = FALSE;
  result->upack_ind.size = size;

  result->upack_ind.lower.ts_type = upack;
  result->upack_ind.lower.indeterminate = FALSE;
  result->upack_ind.lower.base_ts_type = undefined;
  result->upack_ind.lower.modified = FALSE;
  result->upack_ind.lower.size = size;

  result->upack_ind.upper.ts_type = upack;
  result->upack_ind.upper.indeterminate = FALSE;
  result->upack_ind.upper.base_ts_type = undefined;
  result->upack_ind.upper.modified = FALSE;
  result->upack_ind.upper.size = size;

  result->upack_ind.distribution.right_off = 0;
  result->upack_ind.distribution.left_off = 0;
  result->upack_ind.distribution.name = (event->chunked.missing)? MISSING: UNIFORM;

  /* ASSIGN_GRANULES_IND(temp,event->upack_ind.size,event-> */
  /*
  temp = event->chunked.data;
  */
  temp = event->word[1];
  if (event->chunked.sign) {
    /* NEGATIVE_MASK(temp) */
    temp = temp | CHUNKED_STANDARD_MASK;
    }

  result_lower = (event_type)&(result->upack_ind.lower);
  result_upper = (event_type)&(result->upack_ind.upper);

  switch (size) {
    case SIZE32: 
      CONSTRUCT32(temp,temp32);
      ASSIGN32(temp32,result_lower->upack32.granules);
      tadt_e_add_fs(result_lower,
 		    CHUNK32(chunked,event), 
		    result_upper);
      break;
    case SIZE64: 
      CONSTRUCT64(temp,event->word[2],temp64);
      ASSIGN64(temp64,result_lower->upack64.granules);
      tadt_e_add_fs((event_type) &result->upack_ind.lower, 
 		    CHUNK64(chunked,event), 
		    (event_type) &result->upack_ind.upper);
      break;
    case SIZE96: 
      CONSTRUCT96(temp,event->word[2],event->word[3],temp96);
      ASSIGN96(temp96,result_lower->upack96.granules);
      tadt_e_add_fs((event_type) &result->upack_ind.lower, 
 		    CHUNK96(chunked,event), 
		    (event_type) &result->upack_ind.upper);
      break;
    default:
      internal_error("%s", "bad size");
    }
  return tadt_ok;
}

/*  
*  Routine:  tadt_unpack_chunked_nonstandard
*
*  Description:  Unpacks a chunked timestamp with a nonstandard distribution.
*
*  Arguments: event -- (IN): event structure
*             result -- (OUT): result event
*             precision -- (IN): precision info
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

static error_type tadt_unpack_chunked_nonstandard(event_type event, event_type result, precision_type precision)
{
 return tadt_ok;
}

/*  
*  Routine:  tadt_unpack_general
*
*  Description:  Unpacks a general timestamp with a standard distribution.
*
*  Arguments: event -- (IN): event structure
*             result -- (OUT): result event
*             precision -- (IN): precision info
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

static error_type tadt_unpack_general(event_type event, event_type result, precision_type precision)
{
 return tadt_ok;
}

/*  
*  Routine:  tadt_unpack_general_nonstandard
*
*  Description:  Unpacks a general timestamp with a nonstandard distribution.
*
*  Arguments: event -- (IN): event structure
*             result -- (OUT): result event
*             precision -- (IN): precision info
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

static error_type tadt_unpack_general_nonstandard(event_type event, event_type result, precision_type precision)
{
 return tadt_ok;
}

#endif

/*  
*  Routine:  tadt_unpack_determinate
*
*  Description:  Unpacks a basic determinate timestamp into the unpacked format.
*
*  Arguments: event -- (IN): event structure
*             result -- (OUT): result event
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

static error_type tadt_unpack_determinate(event_type event, event_type result, precision_type precision)
{
  unsigned long temp;
  size_type size;
  int32 temp32;
  int64 temp64;
  int96 temp96;

  /* PRECISION_TO_SIZE(precision,size); */
  size = sizes[determinate][precision->precision];

  result->generic.ts_type = upack;
  result->upack.indeterminate = FALSE;
  result->upack.base_ts_type = determinate;
  result->upack.modified = FALSE;
  result->upack.size = size;

  /* ASSIGN_GRANULES(temp,event->upack.size,event-> */
  temp = event->determinate.data;
  if (event->determinate.sign) {
    /* NEGATIVE_MASK(temp) */
    temp = temp | DETERMINATE_MASK;
    }
  switch (size) {
    case SIZE32: 
      result->upack.size = SIZE32;
      CONSTRUCT32(temp,temp32);
      ASSIGN32(temp32,result->upack32.granules);
      break;
    case SIZE64: 
      result->upack.size = SIZE64;
      CONSTRUCT64(temp,event->word[2],temp64);
      ASSIGN64(temp64,result->upack64.granules);
      break;
    case SIZE96: 
      result->upack.size = SIZE96;
      CONSTRUCT96(temp,event->word[2],event->word[3],temp96);
      ASSIGN96(temp96,result->upack96.granules);
      break;
    default:
      internal_error("%s", "bad size");
    }

  return tadt_ok;
}

#ifdef NOWRELATIVE
/*  
*  Routine:  tadt_unpack_nowrelative
*
*  Description:  Unpacks an nowrelative timestamp into the unpacked format.
*
*  Arguments: event -- (IN): event structure
*             result -- (OUT): result event
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

static error_type tadt_unpack_nowrelative_fixedspan(event_type event, event_type result, precision_type precision)
{
  event_space_type temp_event, temp_present;
  unsigned long temp;
  size_type size;
  int32 temp32;
  int64 temp64;
  int96 temp96;

  /* PRECISION_TO_SIZE(precision,size); */
  size = sizes[nowrelative][precision->precision];

  temp_event.generic.ts_type = upack;
  temp_event.upack.indeterminate = FALSE;
  temp_event.upack.base_ts_type = nowrelative;
  temp_event.upack.modified = FALSE;
  temp_event.upack.size = size;

  temp = event->nowrelative.data;
  if (event->nowrelative.sign) {
    temp = temp | NOWRELATIVE_MASK;
    }
  switch (size) {
    case SIZE32: 
      CONSTRUCT32(temp,temp32);
      ASSIGN32(temp32,temp_event.upack32.granules);
      break;
    case SIZE64: 
      CONSTRUCT64(temp,event->word[2],temp64);
      ASSIGN64(temp64,temp_event.upack64.granules);
      break;
    case SIZE96: 
      CONSTRUCT96(temp,event->word[2],event->word[3],temp96);
      ASSIGN96(temp96,temp_event.upack96.granules);
      break;
    default:
      internal_error("%s", "bad size");
    }

  tadt_present_as_of_query(FALSE, upack, precision, &temp_present);
  tadt_e_add_fs(&temp_present, (span_type) &temp_event, result);
  return ok;
}
#endif

/*  
*  Routine:  tadt_unpack_special
*
*  Description:  Unpacks an special timestamp into the unpacked format.
*
*  Arguments: event -- (IN): event structure
*             result -- (OUT): result event
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

static error_type tadt_unpack_special(event_type event, event_type result, precision_type precision)
{

  result->special.ts_type = special; 
  result->special.which = event->special.which;
  return ok;
}

/*  
*  Routine:  tadt_unpack_e
*
*  Description:  Unpacks an event timestamp into the unpacked format.
*
*  Arguments: event -- (IN): event structure
*             result -- (OUT): result event
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_unpack_e(event_type event, event_type result)
{
  switch (event->generic.ts_type) {
    case determinate:/* do not unpack */
      return tadt_unpack_determinate(event,result,&context->precision);

    case upack:/* do not unpack */
      internal_error("%s", "tadt_unpack_e: attempt to unpack an already unpacked event");

    case special: 
      return tadt_unpack_special(event,result,&context->precision);

#ifdef INDETERMINATE
    case indeterminate:
      switch (context->resolution) {
        case chunked:
          return tadt_unpack_chunked(event,result,&context->precision);

        case chunked_nonstandard:
          return tadt_unpack_chunked_nonstandard(event,result,&context->precision);

        case general:
          return tadt_unpack_general(event,result,&context->precision);

        case general_nonstandard:
          return tadt_unpack_general_nonstandard(event,result,&context->precision);

        default:
          internal_error("%s", "tadt_unpack_e: bad indeterminate type ");
      }

#endif

    default:
      internal_error("%s", "tadt_unpack_e: bad passed ts_type");

    }

  return ok;
}

/*  
*  Routine:  tadt_unpack_fs
*
*  Description:  Unpacks a span timestamp into the unpacked format.
*
*  Arguments: span -- (IN): span structure
*             result -- (OUT): result span
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_unpack_fs(span_type span, span_type result) {
   /* exploit event, fixed span similarity */
   return tadt_unpack_e((event_type) span, (event_type) result);
}

/*  
*  Routine:  tadt_unpack_i
*
*  Description:  Unpacks an interval timestamp into the unpacked format.
*
*  Arguments: interval -- (IN): interval structure
*             result -- (OUT): result interval
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_unpack_i(interval_type interval, interval_type result) {
   tadt_unpack_e(interval->start, result->start);
   tadt_unpack_e(interval->terminate, result->terminate);
   return ok;
}

/*  
*  Routine:  tadt_create_e (IS THIS EVER CALLED?)
*
*  Description:  Builds an event timestamp from the given chronon information.
*		 
*
*  Arguments: seconds -- (IN): seconds structure
*             result -- (OUT): result event
*
*  Return Value: Error code
*
*  Errors: out_of_range
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_create_e(seconds_type seconds, event_type result)
{ 
  result->generic.ts_type = seconds->resolution;
  switch (seconds->resolution) {
    case determinate:
      /* Assign data first, then overwrite with timestamp type */
      switch (seconds->size) {
	case SIZE32: ASSIGN32(seconds->seconds32,(int32 *)result); break;
	case SIZE64: ASSIGN64(seconds->seconds64,(int64 *)result); break;
	case SIZE96: ASSIGN96(seconds->seconds96,(int96 *)result); break;
        default: internal_error("%s", "bad size");
	}
      result->determinate.ts_type = determinate;
      break;

    case upack:  
      result->generic.ts_type = upack;
      result->upack.modified = FALSE;
      result->upack.indeterminate = FALSE;
      result->upack.base_ts_type = determinate;
      result->upack.size = seconds->size;
      switch (seconds->size) {
	case SIZE32: ASSIGN32(seconds->seconds32,result->upack32.granules); break;
	case SIZE64: ASSIGN64(seconds->seconds64,result->upack64.granules); break;
	case SIZE96: ASSIGN96(seconds->seconds96,result->upack96.granules); break;
        default: internal_error("%s", "bad size");
	}
      break;

    case special:
	  result->special.which = seconds->special;
      break;

    default:
      internal_error("%s", "tadt_create_e: bad passed ts_type");

    }

  return ok;
}

/*  
*  Routine:  tadt_create_i
*
*  Description:  Builds an interval timestamp from the given events.  Assumes
*     storage for the interval has been pre-allocated.  Also tests the events
*     to make sure that it doesn`t build a bad interval.
*
*  Arguments: event1 -- (IN): starting event
*             event2 -- (IN): terminating event
*             result -- (OUT): result interval
*
*  Return Value: Error code
*
*  Errors: out_of_range
*          bad_interval
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_create_i(event_type event1, event_type event2, interval_type result){
  if (tadt_e_precedes_e(event2,event1)) return bad_interval;
  tadt_copy_e(event1, result->start);
  tadt_copy_e(event2, result->terminate);
  return ok;
}

/*  
*  Routine:  tadt_create_fs
*
*  Description:  Builds a fixed span from the given chronon information.
*
*  Arguments: seconds -- (IN): structure containing span info
*             result -- (OUT): result fixed span
*
*  Return Value: Error code
*
*  Errors: None
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_create_fs(seconds_type seconds, span_type result) {
  /* exploit fixed-span, event similarity */
  return tadt_create_e(seconds, (event_type) result);  
}

/*  
*  Routine:  tadt_create_vs
*
*  Description:  Builds a variable span from the given information.
*
*  Arguments: vs -- (IN):   vspan structure (contains vspan info)
*             result -- (OUT): resulting variable span
*
*  Return Value: Error code
*
*  Errors: None
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_create_vs(vs_type vs, span_type result) {
  result->generic.ts_type = special;
  result->vspan.cal_sys = vs->cal_id;
  result->vspan.dataone = vs->data.word[0];
  result->vspan.datatwo = vs->data.word[1];
  return ok;
}

/*  
*  Routine:  tadt_extract_vs
*
*  Description:  Extract the variable span information from the given vspan.
*
*  Arguments: vs -- (IN):  variable span time-stamp
*             result -- (OUT): resulting variable span information structure
*
*  Return Value: Error code
*
*  Errors: None
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_extract_vs(span_type vs, vs_type result) {
  result->cal_id = vs->vspan.cal_sys;
  result->data.word[0] = vs->vspan.dataone;
  result->data.word[1] = vs->vspan.dataone;
  return ok;
}

/*  
*  Routine:  tadt_is_vs
*
*  Description:  Determine if a span is a variable span.
*
*  Arguments: span -- (IN):  the timestamp to be tested
*
*  Return Value: TRUE if span is a variable span
* 		 FALSE otherwise
*  Errors: None
*  
*  Side Effects: None
*/

bool tadt_is_vs(span_type span) {
  return (span->generic.ts_type == special && span->special.vspan);
}

/*  
*  Routine:  tadt_can_fit_ts
*
*  Description:  Determine if there is enough space allocated to 
*                store a time-stamp.  Time-stamp sizes can be
*                determined using the sizeof function.
*
*  Arguments: ts -- (IN):  the timestamp to be tested 
*             size -- (IN):  how much space is available
*
*  Return Value: TRUE if there is enough space 
* 		 FALSE otherwise
*  Errors: None
*  
*  Side Effects: None
*/

bool tadt_can_fit_ts(timestamp_type ts, int size) {
  /* Just how is this supposed to work? */
  return FALSE; 
}

/*
*  Routine:  tadt_fs_to_seconds
*
*  Description:  Break a fixed span up into its unpacked form (seconds struct)
*
*  Arguments: fs -- (IN):  the fixed span to be converted
*             seconds -- (OUT):  the resulting seconds structure
*
*  Return Value: Error code
* 
*  Errors: None
* 
*  Side Effects: Assignment to passed seconds structure
*/

error_type tadt_fs_to_seconds(span_type fs, seconds_type seconds) {

  return tadt_e_to_seconds((event_type) fs, seconds);
}

/*  
*  Routine:  tadt_e_to_seconds
*
*  Description:  Break an event up into its unpacked form (seconds structure)
*
*  Arguments: event -- (IN):  the event to be converted
*             seconds -- (OUT):  the resulting seconds structure
*
*  Return Value: Error code
*  
*  Errors: None
*  
*  Side Effects: Assignment to passed seconds structure
*/

error_type tadt_e_to_seconds(event_type event, seconds_type seconds) {
  event_space_type result;

  seconds->resolution = event->generic.ts_type;
  switch (event->generic.ts_type) {

    case determinate:
      seconds->special = 0;
      tadt_unpack_e(event,&result);
      switch (result.upack.size) {
	case SIZE32: ASSIGN32(result.upack32.granules,seconds->seconds32); break;
	case SIZE64: ASSIGN64(result.upack64.granules,seconds->seconds64); break;
	case SIZE96: ASSIGN96(result.upack96.granules,seconds->seconds96); break;
        default: internal_error("%s", "bad size");
	}
      return ok;

    case upack:
      seconds->special = 0;
      switch (event->upack.size) {
	case SIZE32: ASSIGN32(event->upack32.granules,seconds->seconds32); break;
	case SIZE64: ASSIGN64(event->upack64.granules,seconds->seconds64); break;
	case SIZE96: ASSIGN96(event->upack96.granules,seconds->seconds96); break;
        default: internal_error("%s", "bad size");
	}
      return ok;

    case special:
      seconds->special = event->special.which;
      return ok;

    default:
      internal_error("%s", "Bad flags");

    }
  return ok;
}

/*  
*  Routine:  tadt_seconds_to_e
*
*  Description:  Convert an unpacked event into its packed form
*
*  Arguments: seconds -- (IN):  the seconds structure to be converted
*             e -- (OUT):  the resulting event
*
*  Return Value: Error code
*  
*  Errors: None
*  
*  Side Effects: Assignment to passed seconds structure
*/

error_type tadt_seconds_to_e(seconds_type seconds, event_type event) {

  return tadt_create_e(seconds, event);

}
